const mysql = require("mysql");
const fs = require('fs');
const mysqldump = require('mysqldump');

const path = require('path');
const cron = require('node-cron'); 
require('dotenv').config({path:__dirname+'/./../../../.env'});

const db = mysql.createPool({
    host: process.env.DB_HOST,
    user: process.env.DB_USER,
    password: process.env.DB_PASS,
    port: process.env.DB_PORT,
    database: process.env.DB_NAME,
    multipleStatements: true,
});


// Configurações do banco
if(process.env.NODE_ENV === "production") {
async function backupDatabase(params) {
    const dbConfig = {
        host: process.env.DB_HOST,
        user: process.env.DB_USER,
        password: process.env.DB_PASS,
        database: process.env.DB_NAME
    };
    
    const today = new Date();
    const formattedDate = today.toISOString().split('T')[0];
    const backupPath = path.join(__dirname, `../../backup/backup-${formattedDate}.sql`);
    
    // Realiza o backup
    mysqldump({
        connection: dbConfig,
        dumpToFile: backupPath,
    }).then(() => {
        console.log(`Backup criado com sucesso em: ${backupPath}`);
    }).catch(err => {
        console.error('Erro ao criar o backup:', err.message);
    });
}


cron.schedule('2 6 * * *', () => {
    console.log('Iniciando backup do banco de dados...');
    backupDatabase();
});
}

console.log(`Connected to database [${process.env.DB_NAME}]`);

global.db = db;

module.exports = db;