const {
    encrypt,
    generateHash,
    checkHash,
  } = require("../../config/auth/cryptography");
  const { genereteToken } = require("../../config/auth/jwt");
  const { decryptDonor } = require("../../util/cryptDonor");
  const db = require("../database/db");
  const nodemailer = require("nodemailer");
  const crypto = require("crypto");
  const path = require("path");
  const bcrypt = require('bcrypt');
  const { validarCPF } = require("../../util/validarCPF")
  const { validarCNPJ } = require("../../util/validarCNPJ")
  const { trocarAno } = require("../../util/dateFormat")
  require("dotenv").config({ path: __dirname + "/./../../../.env" });

  
  
  class produtosDisponiveisRepository {
    constructor() {
      this.db = db;
    }
  
    /**
     * Registra os dados de um usuario
     * @param {*} data dados do usuario
     * @returns mensagem referente ao resultado obtido
     */
    vincularProdutoUser(data) {
        return new Promise(async (resolve, reject) => {
            

            this.db.query(
              `INSERT INTO produtos_disponiveis (id_user, id_produto) VALUES(?,?)`,
              [data.id_user, data.id_produto],
              async (error, response) => {
                if (error) {
                  console.error(error);
                  return resolve({ error: "Erro ao vincular produtos", code: 203 });
                }
                return resolve({ success: "Produtos vinculados com sucesso!", code: 200 });
              }
            );
          
      });
    }
    
   
    /**
     * Lista os dados de um vendedor
     * @returns lista com todos os dados de um usuario
     */
    listProdutoDisponiveisByUser(id_user) {
      return new Promise(async (resolve, reject) => { 
    
  
        try {
          this.db.query(
                    `SELECT 
                      p.id AS id_produto,
                      p.nome AS nome,
                      p.descricao,
                      p.preco1,
                      p.preco2,
                      p.preco3,
                      p.image,
                      p.tipo,
                      p.categoria,
                      p.ordem
                    FROM 
                        produtos_disponiveis pd
                    JOIN 
                        produtos p ON pd.id_produto = p.id 
                    JOIN
                      user u ON pd.id_user = u.id
                    WHERE
                      u.id = ?`,[id_user],
            (error, produtos_disponiveis) => {
              if (error) return reject(new Error(error));
              
              this.db.query(
                `INSERT INTO logs (id_user, data, acao) VALUES (?,?,?)`,
                [id_user, new Date(), "Produtos vinculados"],
                (error, results) => {
                  if (error) {
                    console.error("Error inserting logs: ", error);
                    return reject(error);
                  }
                  resolve();
                }
              );
              return resolve(produtos_disponiveis);
          });
        } catch (error) {
          return reject(new Error(error));
        }
      });
    }

       /**
     * Lista os dados de um vendedor
     * @returns lista com todos os dados de um usuario
     */
       listProdutoComVinculos(id_user) {
        return new Promise(async (resolve, reject) => { 
      
    
          try {
            this.db.query(`SELECT produtos.id AS id_produto, produtos.nome, user.razao_social, user.documento, user.id AS id_user
                        FROM produtos, produtos_disponiveis, user
                        WHERE
                        produtos_disponiveis.id_user = user.id AND
                        produtos_disponiveis.id_produto = produtos.id`,[id_user],
              (error, produtos_disponiveis) => {
                if (error) return reject(new Error(error));
                return resolve(produtos_disponiveis);
            });
          } catch (error) {
            return reject(new Error(error));
          }
        });
      }
  
    /**
     * Desvincula produtos de um usuario especifico 
     * @param {int} id_user ID do usuario a ser desvinculado
     * @param {int} id_produto ID do produto a ser desvinculado
     * @returns mensagem referente ao resultado obtido
     */
    desvincularProdutoDeUsuario(id_user, id_produto) {
      return new Promise((resolve, reject) => {
        this.db.query(
          "DELETE FROM produtos_disponiveis WHERE id_user = ? AND id_produto = ?",
          [id_user, id_produto],
          (error, response) => {
            if (error) {
              return resolve({
                error: "Este produto possui vinculos!",
                code: 203,
              });
            } else {
              return resolve(true);
            }
          }
        );
      });
    }
  
    /**
     * Atualiza os dados de um usuario
     * @param {*} data dados atualizados do usuario
     * @returns mensagem referente ao resultado obtido
     */
    update(data) {
      return new Promise(async (resolve, reject) => {

        let resultado;
        if (data.documento.length == 11) resultado = validarCPF(data.documento)
        if (data.documento.length == 14) resultado = validarCNPJ(data.documento)
        

        if (!resultado) return resolve({ error: `CNPJ ou CPF inexistente!`, code: 203 })

        this.db.query(
          "UPDATE vendedores SET nome=?, email=?, celular=?, documento=?, contrato=?, endereco=? WHERE id=?",
          [data.nome, data.email, data.celular, data.documento, data.contrato, data.endereco, data.id],
          async (error, response) => {
            if (error) return reject(new Error(error));
            else return resolve({ success: "Usuario atualizado com sucesso!" });
          }
        );
      });
    }
  
    /**
     * Lista quantidade minima de produto que um user pode comprar
     * @param {*} id id do usuario
     * @returns quatidade minima de produto de um usuario
     */
    listQtdeMinima(id) {
      return new Promise(async (resolve, reject) => {
        this.db.query(
          `SELECT user.id, user.qtde_minima_suino, user.qtde_minima_bovino FROM user WHERE id = ?`,
          [id],
          async (error, user) => {
            if (error) return reject(new Error(error));
            return resolve(user);
          }
        );
      });
    }

  /**
 * Lista quantidade total de pedidos concluidos por tipos de contratos de vendedor e total de vendas dentro das duas datas fornecidas
 * @param {*} dataInicio Para inicio da filtro
 * @param {*} dataFim Para fim do filtro
 * @returns Todos dias com seus pedidos concluidos se o resultado for 0 significa que nao houve pedido fechado naquele dia
 */
    listQtdeTotalVendedoresContrato(dataInicio, dataFim) {
      return new Promise(async (resolve, reject) => {

        if (!dataInicio || !dataFim) return resolve({ error: `Data não atribuida!`, code: 203 })
        
        dataInicio = trocarAno(dataInicio)
        dataFim = trocarAno(dataFim)

        this.db.query(
          `WITH recursive AllDates AS (
            SELECT DATE(?) AS data -- Início do intervalo
            UNION ALL
            SELECT data + INTERVAL 1 DAY
            FROM AllDates
            WHERE data + INTERVAL 1 DAY <= DATE(?)
          )
          SELECT
              DATE_FORMAT(AllDates.data, "%d/%m") AS data,
              COALESCE(SUM(CASE WHEN vendedores.contrato = 'CLT' THEN pedidos.quantidade ELSE 0 END), 0) AS total_clt,
              COALESCE(SUM(CASE WHEN vendedores.contrato = 'PJ' THEN pedidos.quantidade ELSE 0 END), 0) AS total_pj,
              COALESCE(SUM(CASE WHEN user.id_vendedor IS NULL THEN pedidos.quantidade ELSE 0 END), 0) AS total_sem_vinculo,
              COALESCE(SUM(pedidos.quantidade), 0) AS total_vendas
          FROM 
              AllDates
          LEFT JOIN 
              pedidos ON DATE(pedidos.data) = AllDates.data
          LEFT JOIN 
              user ON pedidos.id_user = user.id
          LEFT JOIN 
              vendedores ON user.id_vendedor = vendedores.id
          WHERE 
              pedidos.concluido = 1 OR pedidos.concluido IS NULL
          GROUP BY 
              AllDates.data
          ORDER BY 
              AllDates.data
            Limit 366`,
          [dataInicio, dataFim],
          async (error, user) => {
            if (error) return reject(new Error(error));
            return resolve(user);
          }
        );
      });
    }
  
}
  
  module.exports = produtosDisponiveisRepository;
  