const cron = require('node-cron');
const db = require("../infra/database/db");
const { sendMessage, sendImage } = require("./gzappy");  

if (process.env.NODE_ENV === "production") {
    async function lembreteDePedido() {
        console.log('Executando a tarefa diária');
        try {
            // Buscar usuários que NÃO fizeram pedidos hoje
            const pedidosNaoFeitos = await new Promise((resolve, reject) => {
                db.query(
                    `SELECT u.*
                    FROM user u
                    LEFT JOIN pedidos p ON u.id = p.id_user AND DATE(p.data) = CURDATE()
                    WHERE p.id_user IS NULL 
                    AND u.status = 'aprovado' 
                    AND u.role != 'ADMIN' 
                    AND u.dias_pedido LIKE CONCAT('%', 
                        CASE DAYOFWEEK(NOW())
                            WHEN 1 THEN 'Domingo'
                            WHEN 2 THEN 'Segunda'
                            WHEN 3 THEN 'Terça'
                            WHEN 4 THEN 'Quarta'
                            WHEN 5 THEN 'Quinta'
                            WHEN 6 THEN 'Sexta'
                            WHEN 7 THEN 'Sábado'
                        END, 
                    '%')`, [],
                    (error, results) => {
                        if (error) return reject(error);
                        resolve(results);
                    }
                );
            });

            // Buscar mensagens vinculadas a usuários com horário agendado
            const lembretes = await new Promise((resolve, reject) => {
                db.query(
                    `SELECT mv.id_user, mv.id_mensagem, mv.horario, u.celular, u.email, u.notificacao_email
                    FROM mensagens_vinculadas mv
                    JOIN user u ON mv.id_user = u.id
                    WHERE u.status = 'aprovado' AND u.role != 'ADMIN'`,
                    [],
                    (error, results) => {
                        if (error) return reject(error);
                        resolve(results);
                    }
                );
            });

            console.log('pedidosNaoFeitos: ', pedidosNaoFeitos)

            // Enviar lembretes para usuários que não fizeram pedidos
            for (const user of pedidosNaoFeitos) {
                const numeroFormatado = `55${user.celular}`;
                const mensagemPadrao = `Olá,\n
Sabemos que hoje é o dia do faturamento da sua empresa, então gostaríamos de lembrá-lo de fazer seus pedidos.\n
Acesse: https://aliancapedidos.com.br e escolha os produtos.`;

                // Se não tiver lembrete vinculado, envia às 15:15
                const usuarioComLembrete = lembretes.some(lembrete => lembrete.id_user === user.id);
                if (!usuarioComLembrete) {
                    cron.schedule('15 15 * * *', async () => {
                        await sendMessage(mensagemPadrao, numeroFormatado);
                        console.log(`Lembrete enviado para ${user.celular} às 15:15`);
                        // if (user.notificacao_email) {
                        //     await enviarEmail(user.email, "Lembrete de Faturamento", "../view/lembrete.html");
                        // }
                    }, {
                        scheduled: true,
                        timezone: "America/Sao_Paulo"
                    });
                }

                // if (user.notificacao_email) {
                //     await enviarEmail(user.email, "Lembrete de Faturamento", "../view/lembrete.html");
                // }
            }

            // Defina a função buscarMensagem no mesmo arquivo
            async function buscarMensagem(id_mensagem) {
            return new Promise((resolve, reject) => {
                db.query(
                    `SELECT mensagem FROM mensagens WHERE id = ?`,
                    [id_mensagem],
                    (error, results) => {
                        if (error) return reject(error);
                        resolve(results[0]?.mensagem);
                    }
                );
            });
            }

            // Função que envia o lembrete
            async function enviarLembrete({ id_user, id_mensagem, celular, email, notificacao_email }) {
            try {
                const mensagem = await buscarMensagem(id_mensagem);
                console.log('mensagem', mensagem);
                if (!mensagem) return;
        
                const numeroFormatado = `55${celular}`;
                await sendMessage(mensagem, numeroFormatado);
                console.log(`Lembrete enviado para usuário ${id_user}`);
        
                // if (notificacao_email) {
                //     await enviarEmail(email, "Lembrete de Faturamento", "../view/lembrete.html");
                // }
            } catch (error) {
                console.error(`Erro ao enviar lembrete para ${id_user}:`, error);
            }
        }
        
        // Função que agendará o lembrete usando cron
        lembretes.forEach(({ id_user, id_mensagem, horario, celular, email, notificacao_email }) => {
            const [hora, minuto] = horario.split(':');
        
            console.log(`Agendando cron para: ${minuto} ${hora} * * *`);
        
            cron.schedule(`${minuto} ${hora} * * *`, () => {
                enviarLembrete({ id_user, id_mensagem, celular, email, notificacao_email });
            });
        
            console.log(`Lembrete agendado para ${celular} às ${horario}`);
        });
        
        

        } catch (error) {
            console.error("Erro ao processar lembretes:", error);
        }
    }

    async function buscarMensagensPromocionais() {
        return new Promise((resolve, reject) => {
            db.query(`
                SELECT id, mensagem, data, hora
                FROM mensagens
                WHERE padrao = false AND data IS NOT NULL AND hora IS NOT NULL
            `, [], (err, results) => {
                if (err) return reject(err);
                resolve(results);
            });
        });
    }
    
    async function buscarUsuariosAprovados() {
        return new Promise((resolve, reject) => {
            db.query(`
                SELECT id, celular, email, notificacao_email
                FROM user
                WHERE status = 'aprovado' AND role != 'ADMIN'
            `, [], (err, results) => {
                if (err) return reject(err);
                resolve(results);
            });
        });
    }
    
    // Sempre que o servidor reiniciar essa função será chamada
    // e agendará todas as mensagens promocionais que estão na tabela mensagens
    async function agendarMensagensPromocionaisParaTodos() {
        try {
            const mensagens = await buscarMensagensPromocionais();
            const usuarios = await buscarUsuariosAprovados();
            
            for (const mensagem of mensagens) {
                const [hora, minuto] = mensagem.hora.split(":");
                const data = new Date(mensagem.data);
                const dia = data.getDate();
                const mes = data.getMonth() + 1; // Janeiro = 0

    
                const cronPattern = `${minuto} ${hora} ${parseInt(dia)} ${parseInt(mes)} *`;
    
                cron.schedule(cronPattern, async () => {
                    for (const user of usuarios) {
                        const numeroFormatado = `55${user.celular}`;
                        await sendMessage(mensagem.mensagem, numeroFormatado);
                        console.log(`Mensagem promocional enviada para ${user.celular}`);
    
                        // if (user.notificacao_email) {
                        //     await enviarEmail(user.email, "Promoção", "../view/promocional.html");
                        // }
                    }
                }, {
                    scheduled: true,
                    timezone: "America/Sao_Paulo"
                });
    
                // console.log(`✅ Mensagem agendada para ${mensagem.data} às ${mensagem.hora}`);
            }
    
        } catch (error) {
            console.error("Erro ao agendar mensagens promocionais:", error);
        }
    }

    // Sempre que a função de criar uma mensagem for chamada
    // essa função será chamada e agendará a mensagem promocional
    // evitando perca de mensagens promocionais criadas depois do servidor reiniciar
    function agendarMensagemPromocional(mensagem) {
        const [ano, mes, dia] = mensagem.data.split("-");
        const [hora, minuto] = mensagem.hora.split(":");
        const cronPattern = `${minuto} ${hora} ${parseInt(dia)} ${parseInt(mes)} *`;
    
        db.query(
            `SELECT celular, email, notificacao_email, notificacao_whats 
             FROM user 
             WHERE status = 'aprovado' AND role != 'ADMIN' AND notificacao_whats = 1`,
            (err, usuarios) => {
                if (err) return console.error("Erro ao buscar usuários:", err);

                console.log('Usuarios: ', usuarios)
    
                cron.schedule(cronPattern, async () => {
                    for (const user of usuarios) {
                        const numeroFormatado = `55${user.celular}`;
                
                        try {
                            if (mensagem.image) {
                                await sendImage(numeroFormatado, mensagem.mensagem, mensagem.image);
                                console.log(`📷 Mensagem com imagem enviada para ${user.celular}`);
                            } else {
                                await sendMessage(mensagem.mensagem, numeroFormatado);
                                console.log(`💬 Mensagem enviada para ${user.celular}`);
                            }
                        } catch (err) {
                            console.error(`❌ Erro ao enviar para ${user.celular}:`, err);
                        }
                    }
                }, {
                    scheduled: true,
                    timezone: "America/Sao_Paulo"
                });
                
    
                console.log(`📅 Mensagem promocional agendada para ${mensagem.data} às ${mensagem.hora}`);
            }
        );
    }

    cron.schedule('00 07 * * *', lembreteDePedido, {
        scheduled: true,
        timezone: "America/Sao_Paulo"
    });

  module.exports = { lembreteDePedido, agendarMensagensPromocionaisParaTodos, agendarMensagemPromocional };
} else {
    console.log("O ambiente não é de produção. Tarefas cron não serão agendadas") 
}
